#import <Rcpp.h>
using namespace Rcpp;


double zjn(double lambda, double ret)
{
	return(ret/exp(lambda/2));
} 

double ujn(double xi, double phi, double d1, double d2, double sigma, double z, double lambda, double logrv)
{
	return((logrv - xi - phi*lambda - d1*(pow(z,2)-1) - d2*z)/sigma);
}

double deltafnjn(double phi, double d1, double d2, double sigma, double z, double u)
{
	return(0.5*(pow(z,2)-1) + (1/sigma)*u*(phi-d1*pow(z,2)-(d2/2)*z));
}

// Leverage
// double transfjn(double omega, double alpha, double S, double Delta, double beta, double lambda, double psi, double z, double jumppar, double jump)
// {
// 	return(omega + alpha*S*Delta + beta*lambda + psi*z + jumppar*log(1+jump));
// }

double transfjn(double omega, double alpha, double S, double Delta, double beta, double lambda, double z, double jumppar, double jump)
{
	return(omega + alpha*S*Delta + beta*lambda + jumppar*log(1+jump));
}

double sfnjn(double phi, double d1, double d2, double sigma)
{
	return 1/(pow(sigma,-2)*(pow(phi,2) + 3*pow(d1,2) + (pow(d2,2)/4) - 2*d1*phi) + 0.5);
}



// [[Rcpp::export]]
List recursive_filter_jumps_n(NumericVector params, NumericVector ret, NumericVector rv, NumericVector jumps)
{
	int n = ret.size();
	double xi = params[0];
	double phi = params[1];
	double d1 = params[2];
	double d2 = params[3];
	double sigma = params[4];
	double omega = params[5];
	double alpha = params[6];
	double beta = params[7];
	// double psi = params[8];
	double jumppar = params[8];

	double sval = sfnjn(phi, d1, d2, sigma);

	std::vector<double> lambda_vals(n);
	std::vector<double> u_vals(n);
	std::vector<double> z_vals(n);

	lambda_vals[0] = params[9];
	z_vals[0] = zjn(lambda_vals[0], ret[0]);
	u_vals[0] = ujn(xi, phi, d1, d2, sigma, z_vals[0], lambda_vals[0], rv[0]);

	for (int i = 1; i < n; i++) {
		lambda_vals[i] = transfjn(omega, alpha, sval, deltafnjn(phi, d1, d2, sigma, z_vals[i-1], u_vals[i-1]), beta, lambda_vals[i-1], z_vals[i-1], jumppar, jumps[i-1]);
		z_vals[i] = zjn(lambda_vals[i], ret[i]);
		u_vals[i] = ujn(xi, phi, d1, d2, sigma, z_vals[i], lambda_vals[i], rv[i]);
	}

	return(List::create( 
	   _["lambda"]  = lambda_vals, 
	   _["u"]  = u_vals, 
	   _["z"] = z_vals
	 ));
}
